/////////////////////////////////////////////////////////////////////////////
//
// DbControl
//
/////////////////////////////////////////////////////////////////////////////
//
// Hakki Dogusan
// dogusanh@tr.net
// http://home.tr.net/dogusanh
//
/////////////////////////////////////////////////////////////////////////////

#include "DbControl.h"
#include "DbQueryDialog.h"


// should be configurable
static const FXchar        CURRENCY_SEPARATOR = ',';
static const FXchar* const CURRENCY_FORMAT    = "%-15.2f";
static const FXint         DECIMAL_POINTS     = 3; //.00
static const FXchar        DATE_SEPARATOR     = '/';
// caution!.. ...hardcoded :-(
// static const FXchar* const DATE_FORMAT = "DD/MM/YYYY";


void CurrencyField::set(const FXString& s)
{
    FXint p;
    FXString ss=s;
    while ((p=ss.findf(CURRENCY_SEPARATOR)) != -1) {
        ss.remove(p);
    }
    currency_=FXDoubleVal(ss);
}

FXString CurrencyField::get() const
{
    FXString s = FXStringFormat(CURRENCY_FORMAT, currency_).trim();
    FXint l = s.length() - DECIMAL_POINTS;
    FXint k = l / 3;
    if ((l % 3) == 0) {
        k -= 1;
    }
    for (FXint i = 1; i <= k; ++i) {
        s.insert(l-i*3, CURRENCY_SEPARATOR);
    }
    return s;
}


void DateField::getDate(FXint& year, FXint& month, FXint& day) const
{
    FXint R, Z, H, A, B, C;
    FXint y, m, d;
	if (date_ < 1 || date_ > 2958464)
	{
		year  = 0;
		month = 0;
		day   = 0;
		return;
	}
    R = date_ + 693595;
    Z = R + 306;
    H = 100*Z - 25;
    A = H/3652425;
    B = A - A/4;
    y = (100*B + H) / 36525;
    C = B + Z - 365*y - y / 4;
    m = (5*C + 456) / 153;
    d = C - (153*m - 457) / 5;
    if (m > 12) { y += 1; m -= 12; }
	year  = y;
	month = m;
	day   = d;
}

void DateField::setDate(FXint year, FXint month, FXint day)
{
    FXint R;
	FXint y, m, d;
	d = day; m = month; y = year;
    if (m < 3) { m += 12; y -= 1; }
    R = d + (153*m - 457) / 5 + 365*y + y/4 - y/100 + y/400 - 306;
    date_ = R - 693595;
	getDate(y, m, d);
	if (y != year || m != month || d != day) date_ = 0;
}

void DateField::set(const FXString& s)
{
    date_=0;
    FXString ss=s;
    ss.trim();
    if (ss.empty()) return;

    FXint vals[3]={0}; //d,m,y
    FXint part=0;
    FXString res;
    do
    {
        res=ss.extract(part,DATE_SEPARATOR);
        if (!res.empty())
        {
            vals[part]=FXIntVal(res);
            ++part;
        }
    }
    while (!res.empty() && part<3);
    setDate(vals[2],vals[1],vals[0]);
}

FXString DateField::get() const
{
	FXint y, m, d;
	// Validity control
	getDate(y, m, d);
	if (y && m && d)
    {
        FXString f("%02d/%02d/%04d");
        return FXStringFormat(f.text(), d, m, y);
    }
    // cannot modify a const object... date_ = 0;
    (const_cast<DateField* const>(this))->date_ = 0;
    return FXString();
}


FXDEFMAP(DbField) DbFieldMap[]={
    FXMAPFUNC(SEL_COMMAND,  DbField::ID_VALUE, DbField::onCmdValue),
    FXMAPFUNC(SEL_CHANGED,  DbField::ID_VALUE, DbField::onCmdValue),
    FXMAPFUNC(SEL_UPDATE,   DbField::ID_VALUE, DbField::onUpdValue),
    FXMAPFUNCS(SEL_COMMAND, DbField::ID_OPTION-10001, DbField::ID_OPTION+10000, DbField::onCmdOption),
    FXMAPFUNCS(SEL_UPDATE,  DbField::ID_OPTION-10001, DbField::ID_OPTION+10000, DbField::onUpdOption),
    
    FXMAPFUNC(SEL_FOCUSIN,  DbField::ID_VALUE, DbField::onFocusIn),
    FXMAPFUNC(SEL_FOCUSOUT, DbField::ID_VALUE, DbField::onFocusOut),
};


FXIMPLEMENT(DbField,FXObject,DbFieldMap,ARRAYNUMBER(DbFieldMap))


void DbField::setFocus()
{
    handle(this,MKUINT(0,SEL_FOCUSIN),NULL);
}

void DbField::killFocus()
{
    handle(this,MKUINT(0,SEL_FOCUSOUT),NULL);
}

long DbField::onFocusIn(FXObject*, FXSelector, void* ptr)
{
    flags_|=FLAG_FOCUSED;
    if(target_) target_->handle(this,MKUINT(message_,SEL_FOCUSIN),ptr);
    return 1;
}

long DbField::onFocusOut(FXObject*, FXSelector, void* ptr)
{
    flags_&=~FLAG_FOCUSED;
    if(target_) target_->handle(this,MKUINT(message_,SEL_FOCUSOUT),ptr);
    return 1;
}

// Value changed from widget
long DbField::onCmdValue(FXObject* sender,FXSelector,void*){
  if (!isEnabled()) return 1;
  if (!isEditable()) return 1;
  FXdouble d;
  FXint    i;
  switch(type_){
    case DT_CHAR:
      i=*((FXchar*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
      *((FXchar*)data_)=(FXchar)i;
      break;
    case DT_UCHAR:
      i=*((FXuchar*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
      *((FXuchar*)data_)=(FXuchar)i;
      break;
    case DT_SHORT:
      i=*((FXshort*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
      *((FXshort*)data_)=(FXshort)i;
      break;
    case DT_USHORT:
      i=*((FXushort*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),(void*)&i);
      *((FXushort*)data_)=(FXushort)i;
      break;
    case DT_INT:
      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),data_);
      break;
    case DT_UINT:
      sender->handle(this,MKUINT(FXWindow::ID_GETINTVALUE,SEL_COMMAND),data_);
      break;
    case DT_FLOAT:
      d=*((FXfloat*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_GETREALVALUE,SEL_COMMAND),(void*)&d);
      *((FXfloat*)data_)=(FXfloat)d;
      break;
    case DT_DOUBLE:
      d=*((FXdouble*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_GETREALVALUE,SEL_COMMAND),(void*)&d);
      *((FXdouble*)data_)=d;
      break;
    case DT_STRING:
      sender->handle(this,MKUINT(FXWindow::ID_GETSTRINGVALUE,SEL_COMMAND),(void*)data_);
      break;
    case DT_DATE:
      {
      FXString s;
      sender->handle(this,MKUINT(FXWindow::ID_GETSTRINGVALUE,SEL_COMMAND),&s);
      *((DateField*)data_)=s;
      }
      break;
    case DT_CURRENCY:
      {
      FXString s;
      sender->handle(this,MKUINT(FXWindow::ID_GETSTRINGVALUE,SEL_COMMAND),&s);
      *((CurrencyField*)data_)=s;
      }
      break;
    }
  if(target_){
    target_->handle(this,MKUINT(message_,SEL_CHANGED),NULL);
    }
  return 1;
  }


// Widget changed from value
long DbField::onUpdValue(FXObject* sender,FXSelector,void*){
  FXdouble d;
  FXint    i;
  switch(type_){
    case DT_CHAR:
      i=*((FXchar*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
      break;
    case DT_UCHAR:
      i=*((FXuchar*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
      break;
    case DT_SHORT:
      i=*((FXshort*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
      break;
    case DT_USHORT:
      i=*((FXushort*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),(void*)&i);
      break;
    case DT_INT:
      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),data_);
      break;
    case DT_UINT:
      sender->handle(this,MKUINT(FXWindow::ID_SETINTVALUE,SEL_COMMAND),data_);
      break;
    case DT_FLOAT:
      d=*((FXfloat*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETREALVALUE,SEL_COMMAND),(void*)&d);
      break;
    case DT_DOUBLE:
      d=*((FXdouble*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETREALVALUE,SEL_COMMAND),(void*)&d);
      break;
    case DT_STRING:
      sender->handle(this,MKUINT(FXWindow::ID_SETSTRINGVALUE,SEL_COMMAND),(void*)data_);
      break;
    case DT_DATE:
      {
      FXString s=*((DateField*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETSTRINGVALUE,SEL_COMMAND),&s);
      }
      break;
    case DT_CURRENCY:
      {
      FXString s=*((CurrencyField*)data_);
      sender->handle(this,MKUINT(FXWindow::ID_SETSTRINGVALUE,SEL_COMMAND),&s);
      }
      break;
    }
  FXuint msg;
  msg=(isEnabled()) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
  sender->handle(this,MKUINT(msg,SEL_COMMAND),NULL);
  //~ msg=(hasFocus()) ? SEL_FOCUSIN : SEL_FOCUSOUT;
  //~ sender->handle(this,MKUINT(0,msg),NULL);
  return 1;
  }


// Value set from message id
long DbField::onCmdOption(FXObject*,FXSelector sel,void*){
  if (!isEnabled()) return 1;
  if (!isEditable()) return 1;
  FXint num=((FXint)SELID(sel))-ID_OPTION;
  switch(type_){
    case DT_CHAR:
      *((FXchar*)data_)=(FXchar)num;
      break;
    case DT_UCHAR:
      *((FXuchar*)data_)=(FXuchar)num;
      break;
    case DT_SHORT:
      *((FXshort*)data_)=(FXshort)num;
      break;
    case DT_USHORT:
      *((FXushort*)data_)=(FXushort)num;
      break;
    case DT_INT:
      *((FXint*)data_)=num;
      break;
    case DT_UINT:
      *((FXuint*)data_)=num;
      break;
    case DT_FLOAT:
      *((FXfloat*)data_)=(FXfloat)num;
      break;
    case DT_DOUBLE:
      *((FXdouble*)data_)=num;
      break;
    }
  if(target_){
    target_->handle(this,MKUINT(message_,SEL_CHANGED),NULL);
    } 
  return 1;
  }


// Check widget whose message id matches
long DbField::onUpdOption(FXObject* sender,FXSelector sel,void*){
  FXint num=((FXint)SELID(sel))-ID_OPTION;
  FXint i=0;
  switch(type_){
    case DT_CHAR:
      i=*((FXchar*)data_);
      break;
    case DT_UCHAR:
      i=*((FXuchar*)data_);
      break;
    case DT_SHORT:
      i=*((FXshort*)data_);
      break;
    case DT_USHORT:
      i=*((FXushort*)data_);
      break;
    case DT_INT:
      i=*((FXint*)data_);
      break;
    case DT_UINT:
      i=*((FXuint*)data_);
      break;
    case DT_FLOAT:
      i=(FXint) *((FXfloat*)data_);
      break;
    case DT_DOUBLE:
      i=(FXint) *((FXdouble*)data_);
      break;
    }
  if(i==num){
    sender->handle(this,MKUINT(FXWindow::ID_CHECK,SEL_COMMAND),NULL);
    }
  else{
    sender->handle(this,MKUINT(FXWindow::ID_UNCHECK,SEL_COMMAND),NULL);
    }
  FXuint msg;
  msg=(isEnabled()) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
  sender->handle(this,MKUINT(msg,SEL_COMMAND),NULL);
  //~ msg=(hasFocus()) ? SEL_FOCUSIN : SEL_FOCUSOUT;
  //~ sender->handle(this,MKUINT(0,msg),NULL);
  return 1;
  }


FXIMPLEMENT(DbKey, FXObject, NULL, 0)


FXDEFMAP(DbControl) DbControlMap[] = {
    FXMAPFUNC(SEL_CHANGED,  DbControl::ID_FIELD,    DbControl::onFieldChanged ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_FIELD,    DbControl::onFieldUpdate  ),
    FXMAPFUNC(SEL_FOCUSIN,  DbControl::ID_FIELD,    DbControl::onFieldFocusIn ),
    FXMAPFUNC(SEL_FOCUSOUT, DbControl::ID_FIELD,    DbControl::onFieldFocusOut),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_OPEN,     DbControl::onCmdOpen      ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_OPEN,     DbControl::onUpdOpen      ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_CLOSE,    DbControl::onCmdClose     ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_CLOSE,    DbControl::onUpdClose     ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_INSERT,   DbControl::onCmdInsert    ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_INSERT,   DbControl::onUpdInsert    ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_KILL,     DbControl::onCmdKill      ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_KILL,     DbControl::onUpdKill      ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_POST,     DbControl::onCmdPost      ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_POST,     DbControl::onUpdPost      ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_CANCEL,   DbControl::onCmdCancel    ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_CANCEL,   DbControl::onUpdCancel    ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_QUERY,    DbControl::onCmdQuery     ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_QUERY,    DbControl::onUpdQuery     ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_FIRST,    DbControl::onCmdFirst     ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_FIRST,    DbControl::onUpdFirst     ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_PRIOR,    DbControl::onCmdPrior     ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_PRIOR,    DbControl::onUpdPrior     ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_NEXT,     DbControl::onCmdNext      ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_NEXT,     DbControl::onUpdNext      ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_LAST,     DbControl::onCmdLast      ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_LAST,     DbControl::onUpdLast      ),
    FXMAPFUNC(SEL_COMMAND,  DbControl::ID_PRINT,    DbControl::onCmdPrint     ),
    FXMAPFUNC(SEL_UPDATE,   DbControl::ID_PRINT,    DbControl::onUpdPrint     ),
};

FXIMPLEMENT(DbControl, FXObject, DbControlMap, ARRAYNUMBER(DbControlMap))


DbControl::DbControl(FXWindow* owner, FXObject* tgt, FXSelector sel)
    : owner_(owner)
    , target_(tgt)
    , message_(sel)
    , flags_(FLAG_ENABLED)
    , mode_(DB_INACTIVE)
    , keysIx_(0)
    , focusedField_(NULL)
    , query_("")
{
}

DbControl::~DbControl()
{
    doClose();
}

DbControl::DbMode DbControl::getMode() const
{
    return mode_;
}

void DbControl::setMode(DbControl::DbMode mode)
{
    mode_=mode;
}

DbKey* DbControl::getKey(FXint i) const
{
    if (i==-1) i=keysIx_;
    if (keys_.no()>0 && i<keys_.no()) return keys_[i];
    return NULL;
}

DbField* DbControl::getField(FXint i) const
{
    if (i==-1) return focusedField_;
    if (fields_.no()>0 && i<fields_.no()) return fields_[i];
    return NULL;
}

DbField* DbControl::getField(const FXString& name) const
{
    if (name=="") return focusedField_;
    for (FXint i=0; i<fields_.no(); ++i)
    {
        if (fields_[i]->getName()==name) return fields_[i];
    }
    return NULL;
}

//void DbControl::error(const FXString& msg)
//{
//    FXMessageBox d(owner_,APPLICATION_NAME,msg,NULL,MBOX_OK|DECOR_TITLE|DECOR_BORDER);
//    d.execute();
//}

FXbool DbControl::isBof() const
{
    return (keys_.no()==0) || (keysIx_==0);
}

FXbool DbControl::isEof() const
{
    return (keys_.no()==0) || (keysIx_==keys_.no()-1);
}

FXbool DbControl::open()
{
    if (getMode()!=DB_INACTIVE) close();
    keys_.clear();
    keysIx_=0;
    clear();
    FXbool result=doOpen();
    if (result) setMode(DB_BROWSE);
    inform(ID_OPEN, (void*)result);
    return result;
}

FXbool DbControl::close()
{
    FXbool result=doClose();
    setMode(DB_INACTIVE);
    inform(ID_CLOSE, (void*)result);
    return result;
}

FXbool DbControl::clear()
{
    return doClear();
}

FXbool DbControl::insert()
{
    setMode(DB_INSERT);
    clear();
    doInsert();
    return TRUE;
}

FXbool DbControl::post()
{
    if (doPost())
    {
        setMode(DB_BROWSE);
        return TRUE;
    }
    return FALSE;
}

FXbool DbControl::cancel()
{
    doCancel();
    setMode(DB_BROWSE);
    return seek();
}

FXbool DbControl::kill()
{
    FXuint answer;
    answer=FXMessageBox::question(owner_,MBOX_YES_NO,"Delete record","Record will be deleted.\nAre you sure?");
    if (answer!=MBOX_CLICKED_YES) return FALSE; //rejected

    if (doKill())
    {
        keys_.remove(keysIx_);
        setMode(DB_BROWSE);
        if (keys_.no()==0) return clear();
        if (keysIx_>keys_.no()-1) keysIx_=keys_.no()-1;
        return seek();
    }
    return FALSE;
}

FXbool DbControl::edit()
{
    setMode(DB_EDIT);
    return doEdit();
}

FXbool DbControl::fetch()
{
    if (!query()) return FALSE;
    if (!doFetch()) return FALSE;
    keysIx_=0;
    if (keys_.no()==0) {
        clear();
        return FALSE;
    }
    return seek();
}

FXbool DbControl::query()
{
    if (doQuery()) return TRUE; // user defined query exist, criteria will be set there
    if (owner_==NULL) return FALSE;
    if (focusedField_==NULL) return FALSE;

    DbQueryDialog d(owner_, focusedField_->getName(), focusedField_->getTitle(), focusedField_->getType());
    if (!d.execute(PLACEMENT_CURSOR)) return FALSE;
    if (d.getQuery()=="") return FALSE;
    query_=d.getQuery();
    return TRUE;
}

FXbool DbControl::seek()
{
    FXbool result=TRUE;
    if ((keys_.no() == 0) || (keysIx_ > keys_.no()-1)) {
        keysIx_ = 0;
        clear();
        result=FALSE;
    } else {
        do {
            if (doSeek(keys_[keysIx_])) {
                result = TRUE;
                break;
            } else {
                keys_.remove(keysIx_);
                if (keys_.no() == 0) {
                    result = clear();
                    break;
                }
                if (keysIx_ > keys_.no()-1) keysIx_ = keys_.no()-1;
            }
        } while (TRUE);
    }
    inform(ID_SEEK, (void*)result);
    return result;
}

FXbool DbControl::first()
{
    if(keys_.no()>0) {
        keysIx_ = 0;
        return seek();
    }
    return FALSE;
}

FXbool DbControl::prior()
{
    if (keys_.no()>0 && keysIx_>0) {
        --keysIx_;
        return seek();
    }
    return FALSE;
}

FXbool DbControl::next()
{
    if ((keys_.no()>0) && (keysIx_<keys_.no()-1)) {
        ++keysIx_;
        return seek();
    }
    return FALSE;
}

FXbool DbControl::last()
{
    if (keys_.no()>0) {
        keysIx_=keys_.no()-1;
        return seek();
    }
    return FALSE;
}

FXbool DbControl::print()
{
    return doPrint();
}

FXbool DbControl::isRejected(FXuint sel, void* ptr)
{
    // Target has chance to object to the proposed change
    // 1:rejected; 0:verified

    // dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...
    // standartlara uygun hale getirilecek
    // dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...
    // if (target_ && target_->handle(this,MKUINT(message_,SEL_???),ptr))
    if (target_ && target_->handle(this,MKUINT(sel,SEL_VERIFY),ptr)) return TRUE;
    return FALSE;
}

FXbool DbControl::inform(FXuint sel, void* ptr)
{
    if ( target_ ) {
        // dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...
        // standartlara uygun hale getirilecek
        // dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...dikkat!...
        // target_->handle(this,MKUINT(message_,SEL_???),ptr);
        target_->handle(this,MKUINT(sel,SEL_CHANGED),ptr);
    }
    return TRUE;
}

long DbControl::onFieldChanged(FXObject* sender, FXSelector, void*)
{
    switch (getMode()) {
        case DB_INSERT:
            // if (isRejected(ID_FIELD, (void*)sender)) return 1;
            inform(ID_FIELD, (void*)sender);
            break;
        case DB_EDIT:
            // if (isRejected(ID_FIELD, (void*)sender)) return 1;
            inform(ID_FIELD, (void*)sender);
            break;
        case DB_BROWSE:
            if (keys_.no()<=0) return 1; // not on a valid record
            if (isRejected(ID_FIELD, (void*)sender)) return 1;
            edit();
            inform(ID_FIELD, (void*)sender);
            break;
        default:
            break;
    }
    return 1;
}

long DbControl::onFieldUpdate(FXObject*, FXSelector, void*)
{
    return 1;
}

long DbControl::onFieldFocusIn(FXObject* sender, FXSelector, void*)
{
    DbField* f=dynamic_cast<DbField*>(sender);
    if (f) {
        focusedField_=f;
        inform(ID_FOCUSIN, (void*)focusedField_);
    }
    return 1;
}

long DbControl::onFieldFocusOut(FXObject* sender, FXSelector, void*)
{
    DbField* f=dynamic_cast<DbField*>(sender);
    if (f) {
        focusedField_=f;
        inform(ID_FOCUSOUT, (void*)focusedField_);
    }
    return 1;
}

long DbControl::onCmdOpen(FXObject*, FXSelector, void*)
{
    open();
    return 1;
}

long DbControl::onUpdOpen(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(getMode()==DB_INACTIVE) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdClose(FXObject*, FXSelector, void*)
{
    close();
    return 1;
}

long DbControl::onUpdClose(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(getMode()!=DB_INACTIVE) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdInsert(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_INSERT, NULL)) return 1;
    FXbool result=insert();
    inform(ID_INSERT, (void*)result);
    return 1;
}

long DbControl::onUpdInsert(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdKill(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_KILL, NULL)) return 1;
    FXbool result=kill();
    inform(ID_KILL, (void*)result);
    return 1;
}

long DbControl::onUpdKill(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE && getKey()!=NULL) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdPost(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_POST, NULL)) return 1;
    FXbool result=post();
    inform(ID_POST, (void*)result);
    return 1;
}

long DbControl::onUpdPost(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()!=DB_BROWSE) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdCancel(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_CANCEL, NULL)) return 1;
    FXbool result=cancel();
    inform(ID_CANCEL, (void*)result);
    return 1;
}

long DbControl::onUpdCancel(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()!=DB_BROWSE) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdQuery(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_QUERY, NULL)) return 1;
    FXbool result=fetch();
    inform(ID_QUERY, (void*)result);
    return 1;
}

long DbControl::onUpdQuery(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE && getField()!=NULL) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdFirst(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_FIRST, NULL)) return 1;
    FXbool result=first();
    inform(ID_FIRST, (void*)result);
    return 1;
}

long DbControl::onUpdFirst(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE && !isBof()) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdPrior(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_PRIOR, NULL)) return 1;
    FXbool result=prior();
    inform(ID_PRIOR, (void*)result);
    return 1;
}

long DbControl::onUpdPrior(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE && !isBof()) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdNext(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_NEXT, NULL)) return 1;
    FXbool result=next();
    inform(ID_NEXT, (void*)result);
    return 1;
}

long DbControl::onUpdNext(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE && !isEof()) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdLast(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_LAST, NULL)) return 1;
    FXbool result=last();
    inform(ID_LAST, (void*)result);
    return 1;
}

long DbControl::onUpdLast(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && getMode()==DB_BROWSE && !isEof()) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}

long DbControl::onCmdPrint(FXObject*, FXSelector, void*)
{
    if (isRejected(ID_PRINT, NULL)) return 1;
    FXbool result=print();
    inform(ID_PRINT, (void*)result);
    return 1;
}

long DbControl::onUpdPrint(FXObject* sender, FXSelector, void* ptr)
{
    FXuint msg=(isEnabled() && (getMode()==DB_BROWSE) && (getKey()!=NULL)) ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
    sender->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
    return 1;
}



//
// EOF
//


