/////////////////////////////////////////////////////////////////////////////
//
// DbControl
//
/////////////////////////////////////////////////////////////////////////////
//
// Hakki Dogusan
// dogusanh@tr.net
// http://home.tr.net/dogusanh
//
/////////////////////////////////////////////////////////////////////////////

#ifndef DbControl_h_
#define DbControl_h_

#include "fx.h"



class CurrencyField
{
public:
    CurrencyField() : currency_(0) {}
    CurrencyField(const CurrencyField& from) : currency_(from.currency_) {}
    CurrencyField(const FXString& s) { set(s); }
    CurrencyField(FXdouble d) : currency_(d){}
    CurrencyField& operator=(const CurrencyField& from) { currency_=from.currency_; return *this; }
    CurrencyField& operator=(const FXString& s) { set(s); return *this; }
    CurrencyField& operator=(const FXdouble d) { currency_=d; return *this; }
    operator FXString() const { return get(); }
    operator FXdouble() const { return currency_; }
    FXString asString() const { return get(); }
    FXdouble asDouble() const { return currency_; }
    ~CurrencyField() {}
private:
    void set(const FXString& s);
    FXString get() const;
    FXdouble currency_;
};


class DateField
{
public:
    DateField() : date_(0){}
    DateField(const DateField& from) : date_(from.date_){}
    DateField(const FXString& s) { set(s); }
    DateField(const FXint i) : date_(i){}
    DateField& operator=(const DateField& from) { date_=from.date_; return *this; }
    DateField& operator=(const FXString& s) { set(s); return *this; }
    DateField& operator=(const FXint i) { date_=i; return *this; }
    operator FXString() const { return get(); }
    operator FXint() const { return date_; }
    FXString asString() const { return get(); }
    FXint    asInt() const { return date_; }
    ~DateField() {}
private:
    void set(const FXString& s);
    FXString get() const;
    void setDate(FXint year, FXint month, FXint day);
    void getDate(FXint& year, FXint& month, FXint& day) const;
    FXint date_;
};


class DbField : public FXObject
{
    FXDECLARE(DbField)
public:
    enum {
        DT_VOID=0,
        DT_CHAR,
        DT_UCHAR,
        DT_SHORT,
        DT_USHORT,
        DT_INT,
        DT_UINT,
        DT_FLOAT,
        DT_DOUBLE,
        DT_STRING,
        DT_DATE,
        DT_CURRENCY,
        DT_LAST
    };
public:
    enum {
        ID_VALUE=1,                   /// Will cause the DbField to ask sender for value
        ID_OPTION=ID_VALUE+10001,     /// ID_OPTION+i will set the value to i where -10000<=i<=10000
        ID_LAST=ID_OPTION+10000
    };
public:
    DbField() 
        : name_(""), title_(""), type_(DT_VOID), data_(NULL), flags_(FLAG_ENABLED), target_(NULL), message_(0) {}
    DbField(const FXString& name,const FXString& title,FXchar& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_CHAR), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXuchar& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_UCHAR), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXshort& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_SHORT), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXushort& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_USHORT), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXint& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_INT), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXuint& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_UINT), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXfloat& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_FLOAT), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXdouble& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_DOUBLE), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,FXString& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_STRING), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,DateField& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_DATE), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    DbField(const FXString& name,const FXString& title,CurrencyField& value,FXObject* tgt=NULL,FXSelector sel=0)
        : name_(name), title_(title), type_(DT_CURRENCY), data_(&value), flags_(FLAG_ENABLED), target_(tgt), message_(sel) {}
    virtual ~DbField(){}
public:    
    FXString getName() const { return name_; }
    FXString getTitle() const { return title_; }
    FXuint getType() const { return type_; }
    void* getData() const { return data_; }
    FXbool isEnabled() const { return (flags_&FLAG_ENABLED)!=0; }
    void enable() { flags_|=FLAG_ENABLED; }
    void disable() { flags_&=~FLAG_ENABLED; }
    FXbool hasFocus() const { return (flags_&FLAG_FOCUSED)!=0; }
    void setFocus();
    void killFocus();
    FXbool isEditable() const { return (flags_&FLAG_READONLY)==0; }
    void setEditable(FXbool edit=TRUE) { if(edit) flags_&=~FLAG_READONLY; else flags_|=FLAG_READONLY; }
public:
    long onCmdValue(FXObject*,FXSelector,void*);
    long onUpdValue(FXObject*,FXSelector,void*);
    long onCmdOption(FXObject*,FXSelector,void*);
    long onUpdOption(FXObject*,FXSelector,void*);
    long onFocusIn(FXObject*,FXSelector,void*);
    long onFocusOut(FXObject*,FXSelector,void*);
private:
    enum {
        FLAG_FOCUSED  = 0x00000002,     // Has focus
        FLAG_READONLY = 0x04000000,     // NOT editable
        FLAG_ENABLED  = 0x00000040      // Able to update
    };
    FXString      name_;
    FXString      title_;
    FXuint        type_;
    void         *data_;
    FXObject     *target_;                 // Target object
    FXSelector    message_;                // Message ID
    FXuint        flags_;                  // State flags
};


class DbKey : public FXObject
{
    FXDECLARE(DbKey)
public:
    enum {
        DT_VOID=0,
        DT_NUMBER,
        DT_STRING
    };
public:
    DbKey() : type_(DT_VOID), number_(0), string_(NULL){}
    DbKey(const FXchar&   value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXuchar&  value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXshort&  value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXushort& value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXint&    value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXuint&   value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXfloat&  value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXdouble& value) : type_(DT_NUMBER), number_(value){}
    DbKey(const FXString& value) : type_(DT_STRING), string_(value){}

    FXuint getType() const { return type_; }

    operator FXchar  () const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXuchar () const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXshort () const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXushort() const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXint   () const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXuint  () const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXfloat () const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXdouble() const { return (type_==DT_NUMBER) ? number_ : 0; }
    operator FXString() const { return (type_==DT_STRING) ? string_ : FXString(); }

    virtual ~DbKey(){}
protected:
    FXuint    type_;
    FXdouble  number_;
    FXString  string_;
};


class DbControl : public FXObject
{
    FXDECLARE(DbControl)
    typedef FXObjectListOf<DbKey> DbKeys;
    typedef FXObjectListOf<DbField> DbFields;
public:
    enum {
        ID_DB_FIRST=50000,
        ID_OPEN,
        ID_CLOSE,
        ID_FIELD,                           // field operation
        ID_FOCUSIN,
        ID_FOCUSOUT,
        ID_INSERT,                          // record operation
        ID_KILL,
        ID_POST,
        ID_CANCEL,
        ID_QUERY,                           // navigation
        ID_SEEK,
        ID_FIRST,
        ID_PRIOR,
        ID_NEXT,
        ID_LAST,
        ID_PRINT,                           // misc operations
        ID_DB_LAST,
    };
public:
    enum DbMode {
        DB_INACTIVE,
        DB_BROWSE,
        DB_INSERT,
        DB_EDIT
    };
public:
    DbControl(FXWindow* owner, FXObject* tgt, FXSelector sel);
    virtual ~DbControl();
public:
    FXbool open();
    FXbool close();
    FXbool clear(); //Blank
    FXbool insert();
    FXbool post();
    FXbool cancel();
    FXbool kill(); //Delete
    FXbool edit();
    FXbool fetch();
    FXbool seek();
    FXbool query();
    FXbool first();
    FXbool prior();
    FXbool next();
    FXbool last();
    FXbool print();
    FXbool isBof() const;
    FXbool isEof() const;
public:
    DbMode getMode() const;
    DbKey* getKey(FXint i=-1) const; // i=-1 => current key
    DbField* getField(FXint i=-1) const; // i=-1 => current focused field
    DbField* getField(const FXString&) const; // NULL => current focused field

    FXString getQuery() const { return query_; }
    void setQuery(const FXString& query) { query_=query; }
    void setTarget(FXObject *t){ target_=t; }
    FXObject* getTarget() const { return target_; }
    void setSelector(FXSelector sel){ message_=sel; }
    FXSelector getSelector() const { return message_; }
    FXbool isEnabled() const { return (flags_&FLAG_ENABLED)!=0; }
    void enable() { flags_|=FLAG_ENABLED; }
    void disable() { flags_&=~FLAG_ENABLED; }
protected:
    void addField(DbField* field) { fields_.append(field); }
    void addField(const FXString& name,const FXString& title,FXchar& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXuchar& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXshort& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXushort& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXint& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXuint& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXfloat& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXdouble& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,FXString& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,DateField& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
    void addField(const FXString& name,const FXString& title,CurrencyField& value) { fields_.append(new DbField(name,title,value,this,ID_FIELD)); }
protected:
    FXWindow* getOwner() const { return owner_; }
    DbKeys& getKeys() { return keys_; }
    DbFields& getFields() { return fields_; }
private:
    virtual FXbool doOpen() { return FALSE; }
    virtual FXbool doClose() { return FALSE; }
    virtual FXbool doClear() { return FALSE; }
    virtual FXbool doInsert() { return FALSE; }
    virtual FXbool doPost() { return FALSE; }
    virtual FXbool doCancel() { return FALSE; }
    virtual FXbool doKill() { return FALSE; }
    virtual FXbool doEdit() { return FALSE; }
    virtual FXbool doFetch() { return FALSE; }
    virtual FXbool doSeek(const DbKey*) { return FALSE; }
    virtual FXbool doQuery() { return FALSE; }
    virtual FXbool doPrint() { return FALSE; }
    virtual FXbool doShowError(const FXString&) { return FALSE; }
public:
    long onFieldChanged(FXObject*, FXSelector, void*);
    long onFieldUpdate(FXObject*, FXSelector, void*);
    long onFieldFocusIn(FXObject*, FXSelector, void*);
    long onFieldFocusOut(FXObject*, FXSelector, void*);
    long onCmdOpen(FXObject*, FXSelector, void*);
    long onUpdOpen(FXObject*, FXSelector, void*);
    long onCmdClose(FXObject*, FXSelector, void*);
    long onUpdClose(FXObject*, FXSelector, void*);
    long onCmdInsert(FXObject*, FXSelector, void*);
    long onUpdInsert(FXObject*, FXSelector, void*);
    long onCmdKill(FXObject*, FXSelector, void*);
    long onUpdKill(FXObject*, FXSelector, void*);
    long onCmdPost(FXObject*, FXSelector, void*);
    long onUpdPost(FXObject*, FXSelector, void*);
    long onCmdCancel(FXObject*, FXSelector, void*);
    long onUpdCancel(FXObject*, FXSelector, void*);
    long onCmdQuery(FXObject*, FXSelector, void*);
    long onUpdQuery(FXObject*, FXSelector, void*);
    long onCmdFirst(FXObject*, FXSelector, void*);
    long onUpdFirst(FXObject*, FXSelector, void*);
    long onCmdPrior(FXObject*, FXSelector, void*);
    long onUpdPrior(FXObject*, FXSelector, void*);
    long onCmdNext(FXObject*, FXSelector, void*);
    long onUpdNext(FXObject*, FXSelector, void*);
    long onCmdLast(FXObject*, FXSelector, void*);
    long onUpdLast(FXObject*, FXSelector, void*);
    long onCmdPrint(FXObject*, FXSelector, void*);
    long onUpdPrint(FXObject*, FXSelector, void*);
private:
    enum {
        FLAG_ENABLED = 0x00000040
    };
    FXuint      flags_;
    FXWindow*   owner_;
    FXObject*   target_;
    FXSelector  message_;
    DbMode      mode_;
    DbKeys      keys_;
    DbFields    fields_;
    FXint       keysIx_;
    DbField*    focusedField_;
    FXString    query_;

    void setMode(DbMode mode);
    FXbool isRejected(FXuint sel, void* ptr);
    FXbool inform(FXuint sel, void* ptr);
protected:
    DbControl(){}
private:
    DbControl(const DbControl&);
    DbControl& operator=(const DbControl&);
};


#endif // DbControl_h
